@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2009-2010 by Internet Systems Consortium, Inc.("ISC")
+ * Copyright(c) 2009-2010,2013 by Internet Systems Consortium, Inc.("ISC")
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -28,8 +28,10 @@
 #include "dhcpd.h"
 
 #include <sys/time.h>
+#include <signal.h>
 
 dhcp_context_t dhcp_gbl_ctx;
+int shutdown_signal = 0;
 
 void
 isclib_cleanup(void)
@@ -217,3 +219,24 @@ isclib_make_dst_key(char          *inname,
 				  &b, dhcp_gbl_ctx.mctx, dstkey));
 }
 
+/**
+ * signal handler that initiates server shutdown
+ *
+ * @param signal signal code that we received
+ */
+void dhcp_signal_handler(int signal) {
+	isc_appctx_t *ctx = dhcp_gbl_ctx.actx;
+	int prev = shutdown_signal;
+
+	if (prev != 0) {
+		/* Already in shutdown. */
+		return;
+	}
+	/* Possible race but does it matter? */
+	shutdown_signal = signal;
+
+	/* Use reload (aka suspend) for easier dispatch() reenter. */
+	if (ctx && ctx->methods && ctx->methods->ctxsuspend) {
+		(void) isc_app_ctxsuspend(ctx);
+	}
+}
