@@ -3,7 +3,8 @@
    Test code for omapip... */
 
 /*
- * Copyright (c) 2004,2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2009-2010,2013-2014 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,22 +25,19 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
+#include "config.h"
+
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
-#include <isc-dhcp/result.h>
+#include <omapip/result.h>
 #include <sys/time.h>
 #include <omapip/omapip.h>
+#include <omapip/isclib.h>
 
 int main (int argc, char **argv)
 {
@@ -47,6 +45,13 @@ int main (int argc, char **argv)
 	omapi_object_t *connection = (omapi_object_t*)0;
 	isc_result_t status;
 
+	status = dhcp_context_create();
+	if (status != ISC_R_SUCCESS) {
+		fprintf(stderr, "Can't initialize context: %s\n",
+			isc_result_totext(status));
+		exit(1);
+	}
+
 	omapi_init ();
 
 	if (argc > 1 && !strcmp (argv [1], "listen")) {
